#ifndef MYTHTUBE_CPP
#define MYTHTUBE_CPP

// C Headers
#include <unistd.h>
#include <math.h>

// QT headers
#include <qapplication.h>
#include <q3network.h>
#include <qdir.h>

//Added by qt3to4:
#include <QUrl>

// MythTV headers
#include <mythtv/libmythui/mythmainwindow.h>
#include <mythtv/util.h>
#include <mythtv/httpcomms.h>
#include <mythtv/mythcontext.h>
#include <mythtv/mythdbcon.h>
#include <mythtv/mythdirs.h>

using namespace std;

#include "mythtube.h"

#include <qfile.h>
#include <qdatastream.h>

// 

//========================================================================================
//              Constructor, Desctructor and setup function
//========================================================================================

MythTube::MythTube(MythScreenStack *parent, const char *name)
    : MythScreenType (parent, name)
{

    // Initialize variables

    m_queryEdit = NULL;
    m_searchButton = NULL;
    m_videoList = NULL;
    m_infoText = NULL;
    m_thumbnailImage = NULL;

    m_progressPopup = NULL;

    httpGrabber = NULL;

    QString fileprefix = GetConfDir();

    QDir dir(fileprefix);
    if (!dir.exists())
        dir.mkdir(fileprefix);
    fileprefix += "/MythTube";
    dir = QDir(fileprefix);
    if (!dir.exists())
        dir.mkdir(fileprefix);


}

MythTube::~MythTube() { }

bool MythTube::Create(){

    bool foundtheme = false;

    foundtheme = LoadWindowFromXML("tube-ui.xml", "tube", this);

    if (!foundtheme)
        return false;

    m_queryEdit = dynamic_cast<MythUITextEdit *> (GetChild("query"));
    m_searchButton = dynamic_cast<MythUIButton *> (GetChild("search"));
    m_videoList = dynamic_cast<MythListButton *> (GetChild("videolist"));
    m_infoText = dynamic_cast<MythUIText *> (GetChild("infotext"));
    m_thumbnailImage = dynamic_cast<MythUIImage *> (GetChild("thumbnail"));

    if (!BuildFocusList())
        VERBOSE(VB_IMPORTANT, "Failed to build a focuslist. Something is wrong");

    m_videoList->SetActive(false);

    connect(m_videoList, SIGNAL(itemSelected( MythListButtonItem*)),
            this, SLOT( updateInfoView(MythListButtonItem*)));
    connect(m_searchButton, SIGNAL(buttonPressed()), this, SLOT(slotSearch()));
    connect(m_videoList, SIGNAL(itemClicked( MythListButtonItem*)),
            this, SLOT( slotViewVideo(MythListButtonItem*)));

    //genera una lista de elementos con foco para poder pasar por ellos

    SetFocusWidget(m_queryEdit);

    return true;
}

bool MythTube::keyPressEvent(QKeyEvent *event)
{
    if (GetFocusWidget()->keyPressEvent(event))
        return true;

    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Tube", event, actions);

    for (int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];
        handled = true;
        if (action == "ESCAPE")
        {
            if (m_progressPopup)
            {
                m_progressPopup->Close();
                m_progressPopup = NULL;
            }

            if (httpGrabber)
                abortHttp = true;
            else
                GetScreenStack()->PopScreen();

        }else{
            handled = false;
        }
    }

    if (!handled && MythScreenType::keyPressEvent(event))
        handled = true;

    return handled;
}



//========================================================================================
//          Slots
//========================================================================================

void MythTube::slotSearch()
{

    QString query = m_queryEdit->GetText();

    Search* search = new Search(query);
    search->execute();
    connect(search, SIGNAL(finished(Search*)),
                this, SLOT(showResult(Search*)));

}

void MythTube::showResult(Search* search){

    search->process();

    m_videoList->Reset();

    for (TubeVideo* video = search->videoList().first(); video;
            video = search->videoList().next()) {

        MythListButtonItem* item =
            new MythListButtonItem(m_videoList, video->title());
        item->setData(video);

    }

}

void MythTube::updateInfoView(MythListButtonItem* selected)
{
    if (!selected)
        return;

    TubeVideo *video = NULL;

    if (m_thumbnailImage)
    {
        m_thumbnailImage->Hide();
    }

     if (GetFocusWidget() == m_videoList)
     {
         video = (TubeVideo*) selected->getData();

        if(video){
             m_infoText -> SetText(video->description());
            if (!video->thumbnail().isEmpty())
            {
                QString fileprefix = GetConfDir();
                fileprefix += "/MythTube";

                QDir dir(fileprefix);
                if (!dir.exists())
                        dir.mkdir(fileprefix);

                fileprefix += "/Tubecache";

                dir = QDir(fileprefix);
                if (!dir.exists())
                    dir.mkdir(fileprefix);

                QString url = video->thumbnail();
                QString sFilename = QString("%1/%2")
                        .arg(fileprefix).arg(qChecksum(url,url.length()));

                bool exists = QFile::exists(sFilename);
                if (!exists)
                    HttpComms::getHttpFile(sFilename, url, 20000);

                if (m_thumbnailImage)
                {
                    m_thumbnailImage->SetFilename(sFilename);
                    m_thumbnailImage->Load();

                    if (!m_thumbnailImage->IsVisible())
                        m_thumbnailImage->Show();
                }
            }
        }
     }
}




//========================================================================================
//          Private functions (Download and play media)
//========================================================================================

void MythTube::createProgress(QString title)
{
    if (m_progressPopup)
        return;

    QString message = title;

    MythScreenStack *popupStack = GetMythMainWindow()->GetStack("popup stack");

    m_progressPopup = new MythUIProgressDialog(message, popupStack,
                                               "mythtubeprogressdialog");

    if (m_progressPopup->Create())
        popupStack->AddScreen(m_progressPopup, false);
}

bool MythTube::getHttpFile(QString sFilename, QString cmdURL)
{
    int redirectCount = 0;
    int timeoutCount = 0;
    QByteArray data(0);
    bool res = false;
    httpGrabber = NULL;
    QString hostname = "";

    createProgress(QObject::tr("Downloading media..."));
    while (1)
    {
        QUrl qurl(cmdURL);
        if (hostname == "")
            hostname = qurl.host();  // hold onto original host

        if (!qurl.hasHost())        // can occur on redirects to partial paths
            qurl.setHost(hostname);

        if (httpGrabber != NULL)
            delete httpGrabber;

        httpGrabber = new HttpComms;
        abortHttp = false;

        httpGrabber->request(qurl, -1, true);

        while ((!httpGrabber->isDone()) && (!abortHttp))
        {
            int total = httpGrabber->getTotal();
            m_progressPopup->SetTotal(total);
            int progress = httpGrabber->getProgress();
            m_progressPopup->SetProgress(progress);
            qApp->processEvents();
            usleep(100000);
        }

        if (abortHttp)
            break;

        // Check for redirection
        if (!httpGrabber->getRedirectedURL().isEmpty())
        {
            if (redirectCount++ < 3)
                cmdURL = httpGrabber->getRedirectedURL();

            // Try again
            timeoutCount = 0;
            continue;
        }

        data = httpGrabber->getRawData();

        if (data.size() > 0)
        {
            QFile file(sFilename);
            if (file.open( QIODevice::WriteOnly ))
            {
                QDataStream stream(& file);
                stream.writeRawBytes( (const char*) (data), data.size() );
                file.close();
                res = true;
            }
        }
        break;
    }

    if (m_progressPopup)
    {
        m_progressPopup->Close();
        m_progressPopup = NULL;
    }

    delete httpGrabber;
    httpGrabber = NULL;
    return res;

}


void MythTube::slotViewVideo(MythListButtonItem* selected){

    if (!selected)
        return;

    TubeVideo *video = (TubeVideo*) selected->getData();
    if(video)
    {
        if (!video->enclosure().isEmpty())
        {
            QString cmdURL(video->enclosure());
            // Handle special cases for media here
            // YouTube: Fetch the mediaURL page and parse out the video URL
            if (cmdURL.contains("youtube.com"))
            {
                cmdURL = QString(video->mediaURL());
                QString mediaPage = HttpComms::getHttp(cmdURL);
                if (!mediaPage.isEmpty())
                {
                    // If this breaks in the future, we are building the URL to download
                    // a video.  At this time, this requires the video_id and the t argument
                    // from the source HTML of the page
                    int playerPos = mediaPage.find("swfArgs") + 7;

                    int tArgStart = mediaPage.find("\"t\": \"", playerPos) + 6;
                    int tArgEnd = mediaPage.find("\"", tArgStart);
                    QString tArgString = mediaPage.mid(tArgStart, tArgEnd - tArgStart);

                    int vidStart = mediaPage.find("\"video_id\": \"", playerPos) + 13;
                    int vidEnd = mediaPage.find("\"", vidStart);
                    QString vidString = mediaPage.mid(vidStart, vidEnd - vidStart);

                    cmdURL = QString("http://youtube.com/get_video.php?video_id=%2&t=%1").arg(tArgString).arg(vidString);
                    VERBOSE(VB_GENERAL, QString("MythNews: VideoURL %1").arg(cmdURL));
                }
            }

            QString fileprefix = GetConfDir();

            QDir dir(fileprefix);
            if (!dir.exists())
                    dir.mkdir(fileprefix);

            fileprefix += "/MythTube";

            dir = QDir(fileprefix);
            if (!dir.exists())
                dir.mkdir(fileprefix);

            QString sFilename(fileprefix + "/newstempfile");

            if (getHttpFile(sFilename, cmdURL))
            {
                qApp->unlock();
                playVideo(sFilename);
                qApp->lock();
            }
        }
        else {
            VERBOSE(VB_GENERAL, QString("no hay video"));
        }
    }

}

void MythTube::playVideo(const QString &filename)
{
    QString command_string = gContext->GetSetting("VideoDefaultPlayer");

    gContext->sendPlaybackStart();

    if ((command_string.find("Internal", 0, false) > -1) ||
        (command_string.length() < 1))
    {
        command_string = "Internal";
        gContext->GetMainWindow()->HandleMedia(command_string, filename);
    }
    else
    {
        if (command_string.contains("%s"))
            command_string = command_string.replace(QRegExp("%s"), filename);

        myth_system(command_string);
    }

    gContext->sendPlaybackEnd();
}

#endif
