#ifndef TUBEVIDEO_CPP
#define TUBEVIDEO_CPP

// #include <iostream>

#include <qfile.h>
#include <qdatastream.h>
#include <qdom.h>
#include <q3urloperator.h>
#include <q3network.h>
#include <q3process.h>

#include "tubevideo.h"

#include <mythtv/mythcontext.h>
#include <mythtv/mythdirs.h>

using namespace std;


//========================================================================================
//          TubeVideo Construction, Destruction
//========================================================================================


TubeVideo::TubeVideo(Search *parent, const QString& title,
                const QString& desc, const QString& tubeURL,
                const QString& thumbnail, const QString& mediaURL,
                const QString& enclosure){

    parent->insertTubeVideo(this);

    m_title = title;
    m_desc = desc;
    m_tubeURL = tubeURL;
    m_thumbnail = thumbnail;
    m_mediaURL = mediaURL;
    m_enclosure = enclosure;

}

TubeVideo::~TubeVideo(){

}




//========================================================================================
//          Search Construction, Destruction
//========================================================================================

Search::Search(const QString& query){

    m_destDir  = GetConfDir();
    m_destDir += "/MythTube";
    m_name    = "MythTube_temp";

    m_query = query;

    q3InitNetworkProtocols();
    m_url = "http://doc.trolltech.com/4.0/q3urloperator.html";
    m_urlOp = new Q3UrlOperator();


    m_videoList.setAutoDelete(true);
    m_videoList.clear();

    connect(m_urlOp, SIGNAL(data(const QByteArray&, Q3NetworkOperation*)),
            this, SLOT(slotGotData(const QByteArray&, Q3NetworkOperation*)));
    connect(m_urlOp, SIGNAL(finished(Q3NetworkOperation*)),
            this, SLOT(slotFinished(Q3NetworkOperation*)));

}

Search::~Search(){
    m_videoList.clear();
}




//========================================================================================
//          Public Function
//========================================================================================

void Search::execute(){

//     Q3Process* proc = new Q3Process("/usr/bin/firefox");
//     proc->start();

    m_urlOp->stop();

    m_state = Search::Retrieving;
    m_data.resize(0);

    QString query = m_query;
    query.replace(QChar(' '),QChar('+'));
    QString searchString = "http://gdata.youtube.com/feeds/api/videos?vq="+query;

    m_url = searchString;

    m_urlOp->get(m_url);

}

void Search::process(){
    QDomDocument domDoc;

    QFile xmlFile(m_destDir+QString("/")+m_name);
    if (!xmlFile.exists()) {
        return;
    }

    if (!xmlFile.open(QIODevice::ReadOnly)) {
        VERBOSE(VB_IMPORTANT, "MythNews: NewsEngine: failed to open xmlfile");
        return;
    }

    if (!domDoc.setContent(&xmlFile)) {
        VERBOSE(VB_IMPORTANT, "MythNews: NewsEngine: failed to set content from xmlfile");
        return;
    }

    parseAtom(domDoc);
    xmlFile.close();
}

void Search::parseAtom(QDomDocument domDoc)
{
    QDomNodeList entries = domDoc.elementsByTagName(QString::fromLatin1("entry"));

    QDomNode itemNode;
    QString title, description, url, thumbnail, mediaurl, enclosure, imageURL;

    for (unsigned int i = 0; i < entries.count(); i++) {
        itemNode = entries.item(i);
        title    = itemNode.namedItem(QString::fromLatin1("title")).toElement().text().simplifyWhiteSpace();
        if (!title.isNull())
            ReplaceHtmlChar(title);

        QDomNode summNode = itemNode.namedItem(QString::fromLatin1("content"));
        if (!summNode.isNull())
        {
            description = summNode.toElement().text().simplifyWhiteSpace();
            ReplaceHtmlChar(description);
        }
        else
            description = QString::null;

        QDomNode linkNode = itemNode.namedItem(QString::fromLatin1("link"));
        if (!linkNode.isNull()){
            QDomAttr linkHref = linkNode.toElement().attributeNode("href");
            if(!linkHref.isNull())
                url = linkHref.value();
        }
        else
            url = QString::null;

        // From this point forward, we process RSS 2.0 media tags.  Please put all
        // other tag processing before this
        // Some media: tags can be enclosed in a media:group item.  If this item is
        // present, use it to find the media tags, otherwise, proceed.
        QDomNode mediaGroup = itemNode.namedItem(QString::fromLatin1("media:group"));
            if (!mediaGroup.isNull())
                itemNode = mediaGroup;

        QDomNode thumbNode = itemNode.lastChildElement(QString::fromLatin1("media:thumbnail"));
        if (!thumbNode.isNull())
        {
            QDomAttr thumburl = thumbNode.toElement().attributeNode("url");
            if (!thumburl.isNull())
                thumbnail = thumburl.value();
        } else
            thumbnail = QString::null;

        QDomNode playerNode = itemNode.namedItem(QString::fromLatin1("media:player"));
        if (!playerNode.isNull())
        {
            QDomAttr mediaURL = playerNode.toElement().attributeNode("url");
            if (!mediaURL.isNull())
               mediaurl  = mediaURL.value();
            // VERBOSE(VB_GENERAL, QString("MythNews: Media URL is %1").arg(mediaurl));
        } else
            mediaurl = QString::null;

        QDomNode contentNode = itemNode.namedItem(QString::fromLatin1("media:content"));
        if (!contentNode.isNull())
        {
            QDomAttr enclosureURL = contentNode.toElement().attributeNode("url");
            if (!enclosureURL.isNull())
                enclosure  = enclosureURL.value();
            else
                enclosure = QString::null;
        }

        new TubeVideo(this, title, description, url, thumbnail, mediaurl, enclosure);

    }
}




//========================================================================================
//          Manage video list
//========================================================================================

void Search::insertTubeVideo(TubeVideo* item)
{
    m_videoList.append(item);
}

TubeVideo::List& Search::videoList()
{
    return m_videoList;
}




//========================================================================================
//          Slots
//========================================================================================

void Search::slotGotData(const QByteArray& data,
                           Q3NetworkOperation* op)
{
    if (op)
    {
        const char *charData = data.data();
        int len = data.count();

        int size = m_data.size();
        m_data.resize(size + len);
        memcpy(m_data.data()+size, charData, len);
    }

}

void Search::slotFinished(Q3NetworkOperation* op)
{
    if (op->state() == Q3NetworkProtocol::StDone &&
        op->errorCode() == Q3NetworkProtocol::NoError) 
    {

        QFile xmlFile(m_destDir+QString("/")+m_name);
        if (xmlFile.open( QIODevice::WriteOnly )) {
            QDataStream stream( &xmlFile );
            stream.writeRawBytes(m_data.data(), m_data.size());
            xmlFile.close();
            m_state = Search::Success;
        }
        else 
        {
            m_state = Search::WriteFailed;
            VERBOSE(VB_IMPORTANT, "MythTube: TubeVideo: Write failed");
        }
    }
    else 
    {
        m_state = Search::RetrieveFailed;
    }

    emit finished(this);
}




//========================================================================================
//          Util
//========================================================================================

void Search::ReplaceHtmlChar(QString &s)
{
    s.replace("&amp;", "&");
    s.replace("&lt;", "<");
    s.replace("&gt;", ">");
    s.replace("&quot;", "\"");
    s.replace("&apos;", "\'");
    s.replace("&#8230;",QChar(8230));
    s.replace("&#233;",QChar(233));
    s.replace("&mdash;", QChar(8212));
    s.replace("&nbsp;", " ");
    s.replace("&#160;", QChar(160));
    s.replace("&#225;", QChar(225));
    s.replace("&#8216;", QChar(8216));
    s.replace("&#8217;", QChar(8217));
    s.replace("&#039;", "\'");
    s.replace("&ndash;", QChar(8211));
    // german umlauts
    s.replace("&auml;", QChar(0x00e4));
    s.replace("&ouml;", QChar(0x00f6));
    s.replace("&uuml;", QChar(0x00fc));
    s.replace("&Auml;", QChar(0x00c4));
    s.replace("&Ouml;", QChar(0x00d6));
    s.replace("&Uuml;", QChar(0x00dc));
    s.replace("&szlig;", QChar(0x00df));
}

#endif
