#ifndef TUBEVIDEO_H
#define TUBEVIDEO_H

#include <qstring.h>
#include <q3ptrlist.h>
#include <qobject.h>
#include <q3cstring.h>

class Q3UrlOperator;
class Q3NetworkOperation;
class Search;
class QDomDocument;

class TubeVideo
{

public:

    typedef Q3PtrList<TubeVideo> List;

    TubeVideo(Search *parent, const QString& title,
                const QString& desc, const QString& tubeURL,
                const QString& thumbnail, const QString& mediaURL,
                const QString& enclosure);
    ~TubeVideo();

    const QString& title() const { return m_title; }
    const QString& description() const { return m_desc; }
    const QString& tubeURL() const { return m_tubeURL; }
    const QString& thumbnail() const { return m_thumbnail; }
    const QString& mediaURL() const { return m_mediaURL; }
    const QString& enclosure() const { return m_enclosure; }

private:

    Search*   m_parent;
    QString   m_title;
    QString   m_desc;
    QString   m_tubeURL;
    QString   m_thumbnail;
    QString   m_mediaURL;
    QString   m_enclosure;

};


class Search : public QObject
{

    Q_OBJECT

public:

    enum State {
        Retrieving = 0,
        RetrieveFailed,
        WriteFailed,
        Success
    };


    Search(const QString& query);
    ~Search();

    void execute();
    void process();
    void parseAtom(QDomDocument domDoc);
    void insertTubeVideo(TubeVideo* item);
    TubeVideo::List& videoList();

private:

    QString        m_query;
    QString        m_name;
    QString        m_destDir;
    QString        m_url;
    Q3UrlOperator* m_urlOp;
    State          m_state;
    QByteArray     m_data;
    TubeVideo::List m_videoList;
    void ReplaceHtmlChar(QString &s);
signals:

    void finished(Search* item);

private slots:

    void slotFinished(Q3NetworkOperation*);
    void slotGotData(const QByteArray& data,
                     Q3NetworkOperation* op);

};


#endif
